import os
import re
from invoke import Context, Config

U_SUCCESS = u'\u2713'
U_FAILURE = u'\u2716'

COLOR_RED = 'red'
COLOR_GREEN = 'green'
COLOR_YELLOW = 'yellow'
COLOR_BLUE = 'blue'
COLOR_PURPLE = 'purple'
COLOR_LIGHTBLUE = 'lightblue'
COLOR_LIGHTGREY = 'lightgrey'
COLOR_GREY = 'grey'
COLOR_PINK = 'pink'


def get_context():
    """
    Get an invoke context object, used to run shell commands.

    :return {invoke.context.Context}: Returns the context object.
    """
    return Context(Config())


def run(cmd, ctx=None, *args, **kwargs):
    if not ctx:
        ctx = get_context()
    return ctx.run(cmd, *args, **kwargs)


def out(msg, color='green', run=True, ctx=None, **kwargs):
    """
    Run an echo command through invoke that prints text in color.

    We will be using the following color codes:

    Black        0;30     Dark Gray     1;30
    Red          0;31     Light Red     1;31
    Green        0;32     Light Green   1;32
    Brown/Orange 0;33     Yellow        1;33
    Blue         0;34     Light Blue    1;34
    Purple       0;35     Light Purple  1;35
    Cyan         0;36     Light Cyan    1;36
    Light Gray   0;37     White         1;37

    :param ctx {invoke.context.Context}: Invoke context variable
    :param msg {str}:
    :param color {str}:
    :param run {bool}:
    :return {NoneType}:
    """
    if not ctx:
        ctx = get_context()

    if os.environ.get('INVOKE_ASCII', False) == '1':
        msg = msg.replace(U_SUCCESS, 'PASS')
        msg = msg.replace(U_FAILURE, 'FAIL')
        msg = msg.encode('utf-8').decode('ascii', 'ignore')

    if run and os.environ.get('INVOKE_NO_COLOR', False) == '1':
        msg = re.sub(r'\s*\$\(tput[a-z0-9 ]+\)\s*', '', msg)
        ctx.run('echo "{}"'.format(msg), **kwargs)
        return msg

    reset = '\033[0m'
    colors = {COLOR_RED: '\033[0;31m',
              COLOR_GREEN: '\033[0;32m',
              COLOR_YELLOW: '\033[0;33m',
              COLOR_BLUE: '\033[0;34m',
              COLOR_PURPLE: '\033[0;35m',
              COLOR_LIGHTBLUE: '\033[1;34m',
              COLOR_LIGHTGREY: '\033[0;37m',
              COLOR_GREY: '\033[1;30m',
              COLOR_PINK: '\033[1;31m', }

    color = '' if (not color or color not in colors) else colors[color]
    msg = '{}{}{}'.format(color, msg, reset)
    if run:
        ctx.run('echo "{}"'.format(msg), **kwargs)
    return msg



import os
from odoo_manager.cli.exceptions import CommandException

pipe_dev_null = '> /dev/null 2>&1'

# We will iterate up the file tree search for a certain trigger file to
# determine where the project base folder is. This is the file that
# we look for.
SEARCH_FILE = 'docker-compose.yml'


class Paths(object):
    base_path = None

    def __init__(self):
        """
        Initialize a Paths object.
        """
        iterated = 0
        working_dir = os.getcwd()

        while not self.base_path and iterated < 100:
            iterated += 1
            if os.path.isfile(os.path.join(working_dir, SEARCH_FILE)):
                self.base_path = working_dir
                break
            working_dir = os.path.abspath(os.path.join(working_dir, '..'))

        if not self.base_path:
            raise CommandException('We cannot find a valid project. Make sure that you are inside of an Odoo project with a docker-compose.yml file.')

    def base(self, path=''):
        """
        Helper method to ensure all paths are relative to tasks.py.

        :param path:
        :return:
        """
        return os.path.join(self.base_path, path)


def iterate_py_files(fn, base):
    """
    :param fn:
    :return {NoneType}:
    """
    ignore = ['.container', '.deployment', '.docs', '.githooks',
              '.monitoring', '.om', '_lib', '_lib_static']
    for dirpath, dirs, filenames in os.walk(base, topdown=True):
        dirs[:] = [d for d in dirs if d not in ignore]
        for filename in [f for f in filenames if f.endswith(".py") and f not in ignore]:
            fn(filename, os.path.join(dirpath, filename))


def iterate_odoo_modules(fn):
    """
    :param fn:
    :return {NoneType}:
    """
    pass



from .base import BaseCommand


class Hello(BaseCommand):
    def __init__(self, options, *args, **kwargs):
        super(Hello, self).__init__(options, depends_on_project=False, *args, **kwargs)

    def run(self):
        print('Hello!')



from invoke import Config, Context
from odoo_manager.core import paths as path_helpers


class BaseCommand(object):
    def __init__(self, options, depends_on_project=True, *args, **kwargs):
        self.options = options
        self.supported_commands = ()
        self.args = args
        self.kwargs = kwargs
        self.ctx = Context(Config())
        self.paths = None
        if depends_on_project:
            self.paths = path_helpers.Paths()

    def get_paths(self):
        if not self.paths:
            self.paths = path_helpers.Paths()
        return self.paths

    def run(self):
        """
        Implements general functionality for the run command. Adds support for
        subcommand via the `supported_commands` tuple.

        Each command class should call super when inheriting the run method.

        ```
        class MyCommand(BaseCommand):
            def run(self):
                subcommand = super(MyCommand, self).run()
                if not subcommand:
                    pass  # Perform some logic
        ```

        :return {NoneType}:
        """
        if not self.supported_commands:
            raise NotImplementedError('All commands must implement the run() method.')

        for command in self.supported_commands:
            if command in self.options and self.options[command]:
                method = 'run_{}'.format(command)
                if not hasattr(self, method):
                    raise NotImplementedError('The function {} has not been implemented yet.'.format(method))
                getattr(self, method)()
                return method

        return None



import os
from .base import BaseCommand
from odoo_manager.core import shell, git
from odoo_manager.core.configs import get_project_manifest
from odoo_manager.core.misc import cleanup

YES_OPTIONS = ('y', 'ye', 'yes', 'ya')


class Setup(BaseCommand):

    def __init__(self, options, *args, **kwargs):
        """
        Construct a Setup command object for `odoo-manager setup`.

        :return {NoneType}:
        """
        super(Setup, self).__init__(options, *args, **kwargs)
        self.supported_commands = ('dependencies', 'mapping', 'version', )

    def run(self):
        """
        Runs when the `odoo-manager setup` command is run.

        :return {NoneType}:
        """
        subcommand = super(Setup, self).run()

        if not subcommand:
            print('setting up')

    def run_dependencies(self):
        """
        Runs when the `odoo-manager setup dependencies` command is run.

        :return {NoneType}:
        """
        options = {'--no-update': self.options.get('--no-update', False),
                   '--only': self.options.get('--only', False),
                   '--yes': self.options.get('--yes', False), }

        paths = self.get_paths()
        configuration = get_project_manifest()
        cleanup()

        if 'depends' in configuration:
            effected_modules = []
            problem_modules = []
            dependencies = options['--only'].split(',') if options['--only'] else configuration['depends']
            for dependency_name in dependencies:
                if not options['--no-update'] or (options['--no-update'] and not os.path.isdir(paths.base('_lib/{}'.format(dependency_name)))):
                    effected_modules.append(dependency_name)
                    git.diff_dependency(self.ctx, dependency_name, configuration['depends'][dependency_name])

            if problem_modules:
                shell.out('The following modules had some problem ...', color='lightred')
                shell.out('  ' + str.join("\n  ", problem_modules), color='lightgrey')

            if effected_modules:
                shell.out('')
                shell.out('Adding the following modules to _lib...', color='lightblue')
                shell.out("\n---\n", color='lightblue')
                shell.out('  ' + str.join("\n  ", effected_modules), color='lightgrey')
                shell.out("\n---\n", color='lightblue')

                if options['--yes'] or str(input('Review any diffs above. Should we continue to overwrite? This will overwrite the _lib modules listed above with files '
                                    'from the dependency source (y/n) :')).lower() in YES_OPTIONS:
                    for dependency_name in effected_modules:
                        git.update_dependency(self.ctx, dependency_name, configuration['depends'][dependency_name])
                shell.out('\nAll done.')
            else:
                shell.out('\nEverything up to date.')

        cleanup()



"""
odoo-manager

Usage:
  odoo-manager hello
  odoo-manager setup [dependencies|mapping|version] [--no-update] [--yes] [--only=<modules>] [-v | --verbose]
  odoo-manager new (project|module) [--name=<project>] [-h | --help] [-v | --verbose]
  odoo-manager -h | --help
  odoo-manager --version

Options:
  --no-update                       Only install new modules instead of updating
                                    existing.
  --yes                             Assume yes to all questions [default: False]
  --only=<modules>                  Only install a certain set of modules
                                    instead of everything in the manifest.
  -h --help                         Show this screen.
  --version                         Show version.
  --name=<project>                  Pass a name for the project.
  -v --verbose                      Enable verbose details.

Examples:
  odoo-manager hello
"""

import traceback
from inspect import getmembers, isclass
from docopt import docopt

import odoo_manager
from odoo_manager.core import shell, configs
from odoo_manager.cli.exceptions import CommandException


def main():
    import odoo_manager.cli.commands
    options = docopt(__doc__, version=odoo_manager.version)
    # configs.init()

    # Here we'll try to dynamically match the command the user is trying to run
    # with a pre-defined command class we've already created.
    for (option_name, option_passed) in options.items():
        if hasattr(odoo_manager.cli.commands, option_name) and option_passed:
            module = getattr(odoo_manager.cli.commands, option_name)
            commands = getmembers(module, isclass)
            command = [command[1] for command in commands if command[0] != 'BaseCommand'][0]
            command = command(options)

            try:
                command.run()

            # We will never show verbose information directly about a
            # CommandException because the user should not need to see the
            # trace on these. These are meant for clear breaking points
            # that we catch and display to the user. These are not unexpected
            # errors.
            #
            # For example, if a project directory cannot be found, this
            # exception will be raised. The user only needs to know that
            # a project directory cannot be found, but not the full stack
            # trace for this error.
            except CommandException as e:
                shell.out('{}\n'.format(str(e)), color=shell.COLOR_RED)

            # These are unexpected error, and we will display trace information
            # to the user to provide them more details about what is going on,
            # if they have the --verbose flag enabled.
            except Exception as e:
                if options.get('--verbose', False):
                    shell.out(traceback.format_exc(), color=shell.COLOR_RED)
                else:
                    shell.out('{}\n'.format(e), color=shell.COLOR_RED)



import odoo_manager
from subprocess import PIPE, Popen as popen
from unittest import TestCase


class TestHelp(TestCase):
    def test_returns_usage_information(self):
        output = popen(['odoo-manager', '-h'], stdout=PIPE).communicate()[0]
        self.assertTrue('Usage:' in output)

        output = popen(['odoo-manager', '--help'], stdout=PIPE).communicate()[0]
        self.assertTrue('Usage:' in output)


class TestVersion(TestCase):
    def test_returns_version_information(self):
        output = popen(['odoo-manager', '--version'], stdout=PIPE).communicate()[0]
        self.assertEqual(output.strip(), odoo_manager.version)



import os
import re
import getpass
from urllib.parse import quote
from .base import BaseCommand
from odoo_manager.core import shell, git


class New(BaseCommand):

    def __init__(self, options, *args, **kwargs):
        """
        Construct a New command object for `odoo-manager new`.

        :return {NoneType}:
        """
        super(New, self).__init__(options, depends_on_project=False, *args, **kwargs)
        self.supported_commands = ('project', 'module', )

    def run_project(self):
        """
        Handles `odoo-manager new project`.

        :return {NoneType}:
        """
        shell.out('Making a new project...')

        verbose = self.options.get('--verbose', False)
        name = self.options.get('--name', False)
        branch = 'master'

        if not name:
            shell.out('Missing project name. Try \"odoo-manager new project --name=my_project\"')
            return

        try:
            shell.run('git clone git@bitbucket.org:bluestingray/collections_scaffold {} --branch="{}" "{}"'.format('--quiet' if not verbose else '', branch, name))
        except:
            username = quote(input("bitbucket username? "))
            password = quote(getpass.getpass("bitbucket password? "))

            try:
                shell.run('git clone https://{}:{}@bitbucket.org/bluestingray/collections_scaffold --quiet --branch="{}" "{}"'.format(
                    username,
                    password,
                    branch,
                    name))
            except:
                shell.out('Invalid credentials.', color=shell.COLOR_RED)

        shell.run('rm -rf "./{}/.git"'.format(name))
        shell.out('Successfully created the project:')
        shell.out('  name:         {}'.format(name), color='lightgrey')

    def run_module(self):
        """
        Handles `odoo-manager new module`.
        """
        verbose = self.options.get('--verbose', False)
        name = input(shell.out(' > What is the technical name of the module? ', color='lightblue', run=False))
        branch = input(shell.out(' > What is the version of the module (9.0, 10.0, 11.0, etc.)? ', color='lightblue', run=False))

        module_display_name = input(shell.out(' > What is the display name of the module? ', color='lightblue', run=False))
        module_category = input(shell.out(' > What is the category of the module? ', color='lightblue', run=False))
        module_tagline = input(shell.out(' > What is a tagline for the module (1 sentence description)? ', color='lightblue', run=False))
        module_summary = input(shell.out(' > What is a summary/short explanation of the module? ', color='lightblue', run=False))

        shell.out('\n')
        shell.out('Generating the module...')

        try:
            shell.run('git clone git@bitbucket.org:bluestingray/module_scaffold {} --branch="{}" "{}"'.format('--quiet' if not verbose else '', branch, name))
        except:
            username = quote(input("bitbucket username? "))
            password = quote(getpass.getpass("bitbucket password? "))

            try:
                shell.run('git clone https://{}:{}@bitbucket.org/bluestingray/module_scaffold --quiet --branch="{}" "{}"'.format(
                    username,
                    password,
                    branch,
                    name))
            except:
                shell.out('Invalid credentials.', color=shell.COLOR_RED)

        shell.run('rm -rf "./{}/.git"'.format(name))

        # Wire the defaults to the module. There are a few things that every
        # module needs:
        #
        # 1. New readme.md file
        # 2. Updated __manifest__.py
        # 3. Updated static/description/index.html
        # 4. Updated doc/index.rst
        with open('./{}/readme.md'.format(name), 'w') as readme_file:
            readme_file.write('# {} ({})'.format(module_display_name, name))
        with open('./{}/static/description/index.html'.format(name), 'w') as index_html_file:
            index_html_file.write('''
<section class="oe_container">
    <div class="oe_row oe_spaced">
        <div class="oe_span12">
            <h2 class="oe_slogan">{}</h2>
            <h3 class="oe_slogan">{}</h3>
        </div>
    </div>
</section>'''.strip('\n').format(module_display_name, module_tagline))
        with open('./{}/doc/index.rst'.format(name), 'w') as index_rst_file:
            content = '{} documentation.'.format(name)
            index_rst_file.write('{}\n{}'.format(content, '=' * len(content)))

        manifest_content = False
        manifest_files = ('__manifest__.py', '__openerp__.py')
        for potential_manifest_file in manifest_files:
            if os.path.isfile('./{}/{}'.format(name, potential_manifest_file)):
                with open('./{}/{}'.format(name, potential_manifest_file), 'r') as manifest_file:
                    manifest_content = manifest_file.read()
                    manifest_content = re.sub(r'[\'"]+name[\'"]+:\s+[\'"]+(.*)[\'"]+,', "'name': '{}',".format(module_display_name), manifest_content)
                    manifest_content = re.sub(r'[\'"]+category[\'"]+:\s+[\'"]+(.*)[\'"]+,', "'category': '{}',".format(module_category), manifest_content)
                    manifest_content = re.sub(r'[\'"]+version[\'"]+:\s+[\'"]+(.*)[\'"]+,', "'version': '{}',".format(branch + '.0'), manifest_content)
                    manifest_content = re.sub(r'[\'"]+summary[\'"]+:\s+[\'"]+(.*)[\'"]+,', "'summary': '{}',".format(module_tagline), manifest_content)
                    manifest_content = re.sub(r'[\'"]+description[\'"]+:\s+[\'"]+(.*)[\'"]+,', "'description': '{}',".format(module_summary), manifest_content)
                if manifest_content:
                    with open('./{}/{}'.format(name, potential_manifest_file), 'w') as manifest_file:
                        manifest_file.write(manifest_content)

        shell.out('Successfully created the module:')
        shell.out('  name:         {}'.format(name), color='lightgrey')
        shell.out('  branch:       {}'.format(branch), color='lightgrey')
        shell.out('  display name: {}'.format(module_display_name), color='lightgrey')
        shell.out('  category:     {}'.format(module_category), color='lightgrey')
        shell.out('  tagline:      {}'.format(module_tagline), color='lightgrey')
        shell.out('  summary:      {}'.format(module_summary), color='lightgrey')



import os
import sys
import filecmp
import getpass
from urllib.parse import quote

from . import configs
from . import paths as path_helpers
from odoo_manager.core import shell

pipe_dev_null = path_helpers.pipe_dev_null


def redirect(verbose=False):
    """
    :param verbose {bool}:
    :return {str}:
    """
    return '' if verbose else (' ' + pipe_dev_null)


class console_colors(object):
    """
    Helper to hold some constant for shell colors.
    """

    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    BOLD = '\033[1m'
    ENDC = '\033[0m'


def print_diff_files(ctx, dcmp):
    """
    :param ctx {invoke.context.Context}: Invoke context variable
    :param dcmp:
    :return {NoneType}:
    """
    for name in dcmp.diff_files:
        print(console_colors.WARNING + "diff_file {} found in {} and {}".format(name, dcmp.left, dcmp.right) + console_colors.ENDC)
    for sub_dcmp in dcmp.subdirs.values():
        print_diff_files(ctx, sub_dcmp)


def _git_clone(ctx, url, branch, output, depth=1):
    """
    Does a system call on git clone, returns the exit status from that system.

    :param url {str}:
    :param branch {str}:
    :return {int}: Returns int representing success or failure
    """
    return ctx.run('git clone {url} {output} --branch={branch} --depth={depth} --quiet{pipe}'.format(url=url,
                                                                                                     branch=branch,
                                                                                                     output=output,
                                                                                                     depth=depth,
                                                                                                     pipe=redirect()))


def _get_git_config():
    git_config = {}
    # right now, just assuming bitbucket.org repo's
    if not configs.config.has_option('options', 'USERNAME') or not configs.config.has_option('options', 'PASSWORD'):
        git_config['username'] = quote(input("bitbucket username? "))
        git_config['password'] = quote(getpass.getpass("bitbucket password? "))
    else:
        git_config['username'] = configs.config.get('options', 'USERNAME')
        git_config['password'] = configs.config.get('options', 'PASSWORD')
    return git_config


def _get_git_urls(repo_url):
    if repo_url[:4] == 'http':
        http_url = repo_url
        ssh_url = repo_url.replace('https://', 'git@')\
            .replace('http://', 'git@')\
            .replace('.com/', '.com:')\
            .replace('.org/', '.org:')
    else:
        ssh_url = repo_url
        http_url = repo_url.replace('git@', 'https://')\
            .replace('.com:', '.com/')\
            .replace('.org:', '.org/')

    return {'ssh': ssh_url, 'http': http_url}


def git_clone(ctx, url, branch, repo_name, output, depth=1):
    urls = _get_git_urls(url)
    try:
        shell.out('        *Trying to clone {}...'.format(urls['ssh']), color='yellow')
        _git_clone(ctx, urls['ssh'], branch, output, depth=depth)
    except:
        try:
            shell.out('        *Trying to clone {}...'.format(urls['http']), color='yellow')
            _git_clone(ctx, urls['http'], branch, output, depth=depth)
        except:
            try:
                credentials = _get_git_config()
                credentials_url = 'https://{username}:{password}@{url}'.format(
                    username=credentials.get('username'),
                    password=credentials.get('password'),
                    url=urls['http'][8:])
                shell.out('        *Trying to clone {}...'.format(credentials_url), color='yellow')
                _git_clone(ctx, credentials_url, branch, output, depth=depth)
            except:
                shell.out('        Sorry, could not clone the repo {}'.format(repo_name), color='red')


def diff_dependency_git(ctx, name, module_config, log=True):
    """
    :param name:
    :param module_config:
    :return: dict Paths for the modules
    """
    paths = path_helpers.Paths()

    if log:
        shell.out('  Processing {}...'.format(name), color='yellow')

    repo_name = module_config['url'].replace('.git', '').split('/').pop()
    repo_temp_path = paths.base('_make_tmp/{}'.format(repo_name))
    original_path = paths.base('_lib/{}'.format(name))

    # If there is no repo cloned down into the temp folder yet, then we are going to git clone it
    # down so that we can access modules that we need to move into _lib.
    if not os.path.isdir(repo_temp_path):
        shell.out('    *Starting git clone for {}...'.format(module_config['url']), color='yellow')

        repo_url = module_config['url']
        repo_branch = module_config['branch']
        git_clone(ctx, repo_url, repo_branch, repo_name, output=paths.base('_make_tmp/{}'.format(repo_name)))

    # Once we've got the modules and files setup, then we are going to check and see if we want
    # to run a diff on these these or not.
    if os.path.isdir(paths.base('{temp_path}/{module_name}'.format(temp_path=repo_temp_path, module_name=name))):
        temp_module_path = paths.base('{temp_path}/{module_name}'.format(temp_path=repo_temp_path, module_name=name))
        if log and os.path.isdir(original_path):
            print_diff_files(ctx, filecmp.dircmp(original_path, temp_module_path))
    else:
        shell.out(' ** Unable to find {module_name} in {repo}.'.format(repo=module_config['url'], module_name=name), color='red')
        exit(1)

    return {'original': original_path, 'updated': temp_module_path}


def diff_dependency(ctx, name, module_config, log=True):
    """
    :param ctx {invoke.context.Context}: Invoke context variable
    :param name {str}:
    :param module_config:
    :return {NoneType}:
    """
    if 'type' in module_config:
        fn = getattr(sys.modules[__name__], 'diff_dependency_{type}'.format(
            type=module_config['type']))
        if fn:
            return fn(ctx, name, module_config, log)
    else:
        return diff_dependency_git(ctx, name, module_config, log)


def update_dependency(ctx, name, module_config):
    """
    :param ctx {invoke.context.Context}: Invoke context variable
    :param name {str}:
    :param module_config:
    :return {NoneType}:
    """
    diff_paths = diff_dependency(ctx, name, module_config, log=False)
    shell.out("  Updating lib with {}...".format(os.path.basename(diff_paths['original'])), color='yellow')
    os.system('rm -rf {}'.format(diff_paths['original']))
    os.system('cp -R {} {}'.format(diff_paths['updated'], diff_paths['original']))



import setuptools
import odoo_manager
from setuptools import Command, find_packages, setup
from subprocess import call


with open("readme.md", "r") as fh:
    long_description = fh.read()


class RunTests(Command):
    description = 'run tests'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        errno = call(['py.test', ])
        raise SystemExit(errno)


setuptools.setup(name="odoo_manager",
                 version=odoo_manager.version,
                 author="Blue Stingray",
                 author_email="odoo@bluestingray.com",
                 description="Odoo manager utility",
                 long_description=long_description,
                 long_description_content_type="text/markdown",
                 install_requires=['docopt', 'invoke==1.1.*', 'configparser==3.7.3', ],
                 packages=setuptools.find_packages(exclude=['docs', 'tests*', ]),
                 python_requires=">=3.3",
                 entry_points={'console_scripts': ['odoo-manager=odoo_manager.cli.cli:main'], },
                 extras_require={'test': ['coverage', 'pytest', 'pytest-cov'], },
                 cmdclass={'test': RunTests},
                 classifiers=["Programming Language :: Python :: 3",
                              "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
                              "Operating System :: OS Independent", ])



import os
import shutil
import configparser
from . import paths as path_helpers


MANIFEST = '__manifest__.py'
LEGACY_MANIFEST = ('__addons_manifest__.py', )


def get_project_manifest():
    """
    Read and parse out the __addons_manifest.py config that sits in the root
    directory of module collection folders.

    This will throw an exception and exit the CLI program / make program all
    together if the file cannot be read properly or the file cannot be parsed
    (exec) properly.

    :raises Exception:
    :return {dict}:
    """
    paths = path_helpers.Paths()

    try:
        f = open(paths.base(MANIFEST), 'r').read()
        return eval(f)
    except Exception as e:
        for legacy_format in LEGACY_MANIFEST:
            if os.path.isfile(paths.base(legacy_format)):
                shutil.move(paths.base(legacy_format), paths.base(MANIFEST))
                try:
                    f = open(paths.base(MANIFEST), 'r').read()
                    return eval(f)
                except Exception as e:
                    continue

        raise Exception('There was a problem processing your __manifest__.py file for the project. Make sure it exists and is in a valid format.' + "\n")


def _parse_config(path):
    """
    Helper method to parse a configuration from a file path.

    :param path {str}: The path to the configuration file
    :return {config.configparser.ConfigParser|NoneType}:
        Returns None if the file cannot be found or if there is an error
        in parsing the configuration file. Otherwise the full configuration
        object is return after performing a `read` on the file.
    """
    try:
        if not os.path.isfile(path):
            return None

        configuration = configparser.ConfigParser(allow_no_value=True)
        configuration.read(path)

        return configuration
    except Exception as e:
        print(str(e))
        return None


def init():
    global setup_calls
    global pipe_dev_null
    global paths
    global standard_env_path
    global odoo_env_path
    global config
    global odoo_env_config

    setup_calls = {}
    pipe_dev_null = path_helpers.pipe_dev_null
    paths = path_helpers.Paths()
    standard_env_path = paths.base('.env')
    odoo_env_path = paths.base('.container/env/odoo/odoo.env')
    config = load_standard_config(standard_env_path)
    odoo_env_config = load_odoo_env_config(odoo_env_path)


def setup_called():
    global setup_calls
    pid = os.getpid()
    ppid = os.getppid()

    setup_calls[ppid] = setup_calls[ppid] + 1 if ppid in setup_calls else 1
    setup_calls[pid] = setup_calls[pid] + 1 if pid in setup_calls else 1


def can_setup():
    global setup_calls
    pid = os.getpid()
    ppid = os.getppid()

    return ppid not in setup_calls and pid not in setup_calls


def load_standard_config(env_path):
    """
    Used to load a standard configuration.

    :param env_path {str}: Path to the configuration file
    :return {config.configparser.ConfigParser|NoneType}:
        Returns None if the file cannot be found or if there is an error
        in parsing the configuration file. Otherwise the full configuration
        object is return after performing a `read` on the file.
    """
    return _parse_config(env_path)


def load_odoo_env_config(env_path):
    """
    Used to load an odoo specific configuration.

    :param env_path {str}: Path to the configuration file
    :return {config.configparser.ConfigParser|NoneType}:
        Returns None if the file cannot be found or if there is an error
        in parsing the configuration file. Otherwise the full configuration
        object is return after performing a `read` on the file.
    """
    return _parse_config(env_path)



import os

def cleanup():
    """
    Cleans up everything related to the make process that runs for this specific
    module collection.

    :return {NoneType}:
    """
    os.system('rm -rf {}'.format('_make_tmp'))
